#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import concrete.uuid.ttypes
import concrete.metadata.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class LinkTarget(object):
  """
  A structure that represents the target of an entity linking annotation.

  Attributes:
   - confidence: Confidence of this LinkTarget object.
   - targetId: A UUID that represents the target of this LinkTarget. This
  UUID should exist in the Entity/Situation(Mention)Set that the
  Linking object is contained in.
   - dbId: A database ID that represents the target of this linking.

  This should be used if the target of the linking is not associated
  with an Entity|Situation(Mention)Set in Concrete, and therefore cannot be linked by
  a UUID internal to concrete.

  If present, other optional field 'dbName' should also be populated.
   - dbName: The name of the database that represents the target of this linking.

  Together with the 'dbId', this can form a pointer to a target
  that is not represented inside concrete.

  Should be populated alongside 'dbId'.
  """

  thrift_spec = (
    None, # 0
    (1, TType.DOUBLE, 'confidence', None, None, ), # 1
    (2, TType.STRUCT, 'targetId', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 2
    (3, TType.STRING, 'dbId', None, None, ), # 3
    (4, TType.STRING, 'dbName', None, None, ), # 4
  )

  def __init__(self, confidence=None, targetId=None, dbId=None, dbName=None,):
    self.confidence = confidence
    self.targetId = targetId
    self.dbId = dbId
    self.dbName = dbName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.DOUBLE:
          self.confidence = iprot.readDouble();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.targetId = concrete.uuid.ttypes.UUID()
          self.targetId.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.dbId = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.dbName = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('LinkTarget')
    if self.confidence is not None:
      oprot.writeFieldBegin('confidence', TType.DOUBLE, 1)
      oprot.writeDouble(self.confidence)
      oprot.writeFieldEnd()
    if self.targetId is not None:
      oprot.writeFieldBegin('targetId', TType.STRUCT, 2)
      self.targetId.write(oprot)
      oprot.writeFieldEnd()
    if self.dbId is not None:
      oprot.writeFieldBegin('dbId', TType.STRING, 3)
      oprot.writeString(self.dbId.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.dbName is not None:
      oprot.writeFieldBegin('dbName', TType.STRING, 4)
      oprot.writeString(self.dbName.encode('utf-8'))
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.confidence)
    value = (value * 31) ^ hash(self.targetId)
    value = (value * 31) ^ hash(self.dbId)
    value = (value * 31) ^ hash(self.dbName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Link(object):
  """
  A structure that represents the origin of an entity linking annotation.

  Attributes:
   - sourceId: The "root" of this Link; points to a EntityMention UUID, Entity UUID, etc.
   - linkTargetList: A list of LinkTarget objects that this Link contains.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'sourceId', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 1
    (2, TType.LIST, 'linkTargetList', (TType.STRUCT,(LinkTarget, LinkTarget.thrift_spec)), None, ), # 2
  )

  def __init__(self, sourceId=None, linkTargetList=None,):
    self.sourceId = sourceId
    self.linkTargetList = linkTargetList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.sourceId = concrete.uuid.ttypes.UUID()
          self.sourceId.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.linkTargetList = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = LinkTarget()
            _elem5.read(iprot)
            self.linkTargetList.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Link')
    if self.sourceId is not None:
      oprot.writeFieldBegin('sourceId', TType.STRUCT, 1)
      self.sourceId.write(oprot)
      oprot.writeFieldEnd()
    if self.linkTargetList is not None:
      oprot.writeFieldBegin('linkTargetList', TType.LIST, 2)
      oprot.writeListBegin(TType.STRUCT, len(self.linkTargetList))
      for iter6 in self.linkTargetList:
        iter6.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.sourceId is None:
      raise TProtocol.TProtocolException(message='Required field sourceId is unset!')
    if self.linkTargetList is None:
      raise TProtocol.TProtocolException(message='Required field linkTargetList is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.sourceId)
    value = (value * 31) ^ hash(self.linkTargetList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Linking(object):
  """
  A structure that represents entity linking annotations.

  Attributes:
   - metadata: Metadata related to this Linking object.
   - linkList: A list of Link objects that this Linking object contains.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'metadata', (concrete.metadata.ttypes.AnnotationMetadata, concrete.metadata.ttypes.AnnotationMetadata.thrift_spec), None, ), # 1
    (2, TType.LIST, 'linkList', (TType.STRUCT,(Link, Link.thrift_spec)), None, ), # 2
  )

  def __init__(self, metadata=None, linkList=None,):
    self.metadata = metadata
    self.linkList = linkList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
          self.metadata.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.LIST:
          self.linkList = []
          (_etype10, _size7) = iprot.readListBegin()
          for _i11 in xrange(_size7):
            _elem12 = Link()
            _elem12.read(iprot)
            self.linkList.append(_elem12)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Linking')
    if self.metadata is not None:
      oprot.writeFieldBegin('metadata', TType.STRUCT, 1)
      self.metadata.write(oprot)
      oprot.writeFieldEnd()
    if self.linkList is not None:
      oprot.writeFieldBegin('linkList', TType.LIST, 2)
      oprot.writeListBegin(TType.STRUCT, len(self.linkList))
      for iter13 in self.linkList:
        iter13.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.metadata is None:
      raise TProtocol.TProtocolException(message='Required field metadata is unset!')
    if self.linkList is None:
      raise TProtocol.TProtocolException(message='Required field linkList is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.metadata)
    value = (value * 31) ^ hash(self.linkList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#
# Autogenerated by Thrift Compiler (1.0.0-dev)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import concrete.communication.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None








from concrete.util import read_communication_from_file


def read_test_comm():
    communication_filename = "tests/testdata/serif_dog-bites-man.concrete"
    return read_communication_from_file(communication_filename)



"""Code for reading and writing Concrete Communications
"""

import cStringIO
import gzip
import mimetypes
import os.path
import tarfile
import zipfile

from thrift import TSerialization
from thrift.protocol import TCompactProtocol
from thrift.transport import TTransport

from concrete import Communication, TokenLattice
from concrete.util.references import add_references_to_communication


def read_thrift_from_file(thrift_obj, filename):
    """Instantiate Thrift object from contents of named file

    The Thrift file is assumed to be encoded using TCompactProtocol

    **WARNING** - Thrift deserialization tends to fail silently.  For
    example, the Thrift libraries will not complain if you try to
    deserialize data from the file `/dev/urandom`.

    Args:

    - `thrift_obj`: A Thrift object (e.g. a Communication object)
    - `filename`:  A filename string

    Returns:

    -  The Thrift object that was passed in as an argument
    """
    thrift_file = open(filename, "rb")
    thrift_bytes = thrift_file.read()
    TSerialization.deserialize(thrift_obj, thrift_bytes, protocol_factory=TCompactProtocol.TCompactProtocolFactory())
    thrift_file.close()
    return thrift_obj

def read_communication_from_file(communication_filename):
    """Read a Communication from the file specified by filename

    Args:

    - `communication_filename`: String with filename

    Returns:

    - A Concrete `Communication` object
    """
    comm = read_thrift_from_file(Communication(), communication_filename)
    add_references_to_communication(comm)
    return comm

def read_tokenlattice_from_file(tokenlattice_filename):
    """
    Takes the filename of a serialized Concrete TokenLattice file,
    reads the TokenLattice from the file and returns an instantiated
    TokenLattice instance.
    """
    return read_thrift_from_file(TokenLattice(), tokenlattice_filename)

def write_communication_to_file(communication, communication_filename):
    return write_thrift_to_file(communication, communication_filename)

def write_thrift_to_file(thrift_obj, filename):
    thrift_bytes = TSerialization.serialize(thrift_obj, protocol_factory=TCompactProtocol.TCompactProtocolFactory())
    thrift_file = open(filename, "wb")
    thrift_file.write(thrift_bytes)
    thrift_file.close()


class CommunicationReader:
    """Iterator/generator class for reading one or more Communications from a file

    The iterator returns a `(Communication, filename)` tuple

    Supported filetypes are:

    - a file with a single Communication
    - a file with multiple Communications concatenated together
    - a gzipped file with a single Communication
    - a gzipped file with multiple Communications concatenated together
    - a .tar.gz file with one or more Communications
    - a .zip file with one or more Communications

    -----

    Sample usage:

        for (comm, filename) in CommunicationReader('multiple_comms.tar.gz'):
            do_something(comm)
    """
    def __init__(self, filename):
        self._source_filename = filename
        if tarfile.is_tarfile(filename):
            # File is either a '.tar' or '.tar.gz' file
            self.filetype = 'tar'
            self.tar = tarfile.open(filename)
        elif zipfile.is_zipfile(filename):
            self.filetype = 'zip'
            self.zip = zipfile.ZipFile(filename, 'r')
            self.zip_infolist = self.zip.infolist()
            self.zip_infolist_index = 0
        elif mimetypes.guess_type(filename)[1] == 'gzip':
            self.filetype = 'stream'
            f = gzip.open(filename, 'rb')
        else:
            self.filetype = 'stream'
            f = open(filename, 'rb')

        if self.filetype is 'stream':
            self.transport = TTransport.TFileObjectTransport(f)
            self.protocol = TCompactProtocol.TCompactProtocol(self.transport)
            self.transport.open()

    def __iter__(self):
        return self

    def next(self):
        """Returns a `(Communication, filename)` tuple

        If the CommunicationReader is reading from an archive, then
        `filename` will be set to the name of the Communication file in
        the archive (e.g. `foo.concrete`), and not the name of the archive
        file (e.g. `bar.zip`).  If the CommunicationReader is reading from
        a concatenated file (instead of an archive), then all
        Communications extracted from the concatenated file will have the
        same value for the `filename` field.
        """
        if self.filetype is 'stream':
            return self._next_from_stream()
        elif self.filetype is 'tar':
            return self._next_from_tar()
        elif self.filetype is 'zip':
            return self._next_from_zip()

    def _next_from_stream(self):
        try:
            comm = Communication()
            comm.read(self.protocol)
            add_references_to_communication(comm)
            return (comm, self._source_filename)
        except EOFError:
            self.transport.close()
            raise StopIteration

    def _next_from_tar(self):
        while True:
            tarinfo = self.tar.next()
            if tarinfo is None:
                raise StopIteration
            if not tarinfo.isfile():
                # Ignore directories
                continue
            filename = os.path.split(tarinfo.name)[-1]
            if filename[0] is '.' and filename[1] is '_':
                # Ignore attribute files created by OS X tar
                continue
            comm = TSerialization.deserialize(
                Communication(),
                self.tar.extractfile(tarinfo).read(),
                protocol_factory=TCompactProtocol.TCompactProtocolFactory())
            add_references_to_communication(comm)
            return (comm, filename)

    def _next_from_zip(self):
        if self.zip_infolist_index >= len(self.zip_infolist):
            raise StopIteration
        zipinfo = self.zip_infolist[self.zip_infolist_index]
        self.zip_infolist_index += 1
        comm = TSerialization.deserialize(
            Communication(),
            self.zip.open(zipinfo).read(),
            protocol_factory=TCompactProtocol.TCompactProtocolFactory())
        add_references_to_communication(comm)
        return (comm, zipinfo.filename)


class CommunicationWriter:
    """Class for writing one or more Communications to a file

    -----

    Sample usage:

        writer = CommunicationWriter('foo.concrete')
        writer.write(existing_comm_object)
        writer.close()
    """
    def close(self):
        self.file.close()

    def open(self, filename):
        self.file = open(filename, 'wb')

    def write(self, comm):
        thrift_bytes = TSerialization.serialize(comm, protocol_factory=TCompactProtocol.TCompactProtocolFactory())
        self.file.write(thrift_bytes)


class CommunicationWriterTGZ:
    """Class for writing one or more Communications to a .TAR.GZ archive

    -----

    Sample usage:

        writer = CommunicationWriterTGZ('multiple_comms.tgz')
        writer.write(comm_object_one, 'comm_one.concrete')
        writer.write(comm_object_two, 'comm_two.concrete')
        writer.write(comm_object_three, 'comm_three.concrete')
        writer.close()
    """
    def close(self):
        self.tarfile.close()

    def open(self, tar_filename):
        self.tarfile = tarfile.open(tar_filename, 'w:gz')

    def write(self, comm, comm_filename):
        thrift_bytes = TSerialization.serialize(comm, protocol_factory=TCompactProtocol.TCompactProtocolFactory())

        file_like_obj = cStringIO.StringIO(thrift_bytes)

        comm_tarinfo = tarfile.TarInfo()
        comm_tarinfo.name = comm_filename
        comm_tarinfo.size = len(thrift_bytes)

        self.tarfile.addfile(comm_tarinfo, file_like_obj)



#!/usr/bin/env python

"""
"""

import time
import unittest

from concrete import AnnotationMetadata, Communication
from concrete.util import generate_UUID
from concrete.validate import validate_communication

class TestUUID(unittest.TestCase):
    def test_generate_uuid(self):
        comm = Communication()
        comm.uuid = generate_UUID()

    def test_minimal_communication_with_uuid(self):
        comm = Communication()
        comm.id = "myID"
        comm.metadata = AnnotationMetadata(tool="TEST", timestamp=int(time.time()))
        comm.type = "Test Communication"
        comm.uuid = generate_UUID()
        self.assertTrue(validate_communication(comm))


if __name__ == '__main__':
    unittest.main(buffer=True)



#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import concrete.metadata.ttypes
import concrete.uuid.ttypes


from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class LanguageIdentification(object):
  """
  A theory about what languages are present in a given communication
  or piece of communication.  Note that it is possible to have more
  than one language present in a given communication.

  Attributes:
   - uuid: Unique identifier for this language identification.
   - metadata: Information about where this language identification came from.
   - languageToProbabilityMap: A list mapping from a language to the probability that that
  language occurs in a given communication.  Each language code should
  occur at most once in this list.  The probabilities do <i>not</i>
  need to sum to one -- for example, if a single communication is known
  to contain both English and French, then it would be appropriate
  to assign a probability of 1 to both langauges.  (Manually
  annotated LanguageProb objects should always have probabilities
  of either zero or one; machine-generated LanguageProbs may have
  intermediate probabilities.)

  Note: The string key should represent the ISO 639-3 three-letter code.
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'uuid', (concrete.uuid.ttypes.UUID, concrete.uuid.ttypes.UUID.thrift_spec), None, ), # 1
    (2, TType.STRUCT, 'metadata', (concrete.metadata.ttypes.AnnotationMetadata, concrete.metadata.ttypes.AnnotationMetadata.thrift_spec), None, ), # 2
    (3, TType.MAP, 'languageToProbabilityMap', (TType.STRING,None,TType.DOUBLE,None), None, ), # 3
  )

  def __init__(self, uuid=None, metadata=None, languageToProbabilityMap=None,):
    self.uuid = uuid
    self.metadata = metadata
    self.languageToProbabilityMap = languageToProbabilityMap

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.uuid = concrete.uuid.ttypes.UUID()
          self.uuid.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.metadata = concrete.metadata.ttypes.AnnotationMetadata()
          self.metadata.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.MAP:
          self.languageToProbabilityMap = {}
          (_ktype1, _vtype2, _size0 ) = iprot.readMapBegin()
          for _i4 in xrange(_size0):
            _key5 = iprot.readString().decode('utf-8')
            _val6 = iprot.readDouble();
            self.languageToProbabilityMap[_key5] = _val6
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('LanguageIdentification')
    if self.uuid is not None:
      oprot.writeFieldBegin('uuid', TType.STRUCT, 1)
      self.uuid.write(oprot)
      oprot.writeFieldEnd()
    if self.metadata is not None:
      oprot.writeFieldBegin('metadata', TType.STRUCT, 2)
      self.metadata.write(oprot)
      oprot.writeFieldEnd()
    if self.languageToProbabilityMap is not None:
      oprot.writeFieldBegin('languageToProbabilityMap', TType.MAP, 3)
      oprot.writeMapBegin(TType.STRING, TType.DOUBLE, len(self.languageToProbabilityMap))
      for kiter7,viter8 in self.languageToProbabilityMap.items():
        oprot.writeString(kiter7.encode('utf-8'))
        oprot.writeDouble(viter8)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.uuid is None:
      raise TProtocol.TProtocolException(message='Required field uuid is unset!')
    if self.metadata is None:
      raise TProtocol.TProtocolException(message='Required field metadata is unset!')
    if self.languageToProbabilityMap is None:
      raise TProtocol.TProtocolException(message='Required field languageToProbabilityMap is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.uuid)
    value = (value * 31) ^ hash(self.metadata)
    value = (value * 31) ^ hash(self.languageToProbabilityMap)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#!/usr/bin/env python

"""
"""

import os
import tarfile
import tempfile
import unittest

from concrete.util import (
    CommunicationReader,
    CommunicationWriter,
    CommunicationWriterTGZ,
    read_communication_from_file
)
from concrete.validate import validate_communication

class TestCommunicationReader(unittest.TestCase):
    def test_single_file(self):
        filename = u'tests/testdata/simple_1.concrete'
        reader = CommunicationReader(filename)
        (comm1, comm1_filename) = reader.next()
        self.assertEqual(u'one', comm1.id)
        self.assertEqual(filename, comm1_filename)

    def test_single_gz_file(self):
        filename = u'tests/testdata/simple_1.concrete.gz'
        reader = CommunicationReader(filename)
        (comm1, comm1_filename) = reader.next()
        self.assertEqual(u'one', comm1.id)
        self.assertEqual(filename, comm1_filename)

    def test_concatenated_file(self):
        filename = u'tests/testdata/simple_concatenated'
        reader = CommunicationReader(filename)
        [comms, filenames] = zip(*[(c, f) for (c, f) in reader])
        for (i, comm_id) in enumerate([u'one', u'two', u'three']):
            self.assertEqual(comm_id, comms[i].id)
            self.assertEqual(filename, filenames[i])

    def test_concatenated_gz_file(self):
        filename = u'tests/testdata/simple_concatenated.gz'
        reader = CommunicationReader(filename)
        [comms, filenames] = zip(*[(c, f) for (c, f) in reader])
        for (i, comm_id) in enumerate([u'one', u'two', u'three']):
            self.assertEqual(comm_id, comms[i].id)
            self.assertEqual(filename, filenames[i])

    def test_tar_file(self):
        filename = u'tests/testdata/simple.tar'
        reader = CommunicationReader(filename)
        [comms, filenames] = zip(*[(c, f) for (c, f) in reader])
        self.assertEqual(u'one', comms[0].id)
        self.assertEqual(u'two', comms[1].id)
        self.assertEqual(u'three', comms[2].id)
        self.assertEqual(u'simple_1.concrete', filenames[0])
        self.assertEqual(u'simple_2.concrete', filenames[1])
        self.assertEqual(u'simple_3.concrete', filenames[2])

    def test_tar_gz_file(self):
        reader = CommunicationReader("tests/testdata/simple.tar.gz")
        [comms, filenames] = zip(*[(c, f) for (c, f) in reader])
        self.assertEqual(u'one', comms[0].id)
        self.assertEqual(u'two', comms[1].id)
        self.assertEqual(u'three', comms[2].id)
        self.assertEqual(u'simple_1.concrete', filenames[0])
        self.assertEqual(u'simple_2.concrete', filenames[1])
        self.assertEqual(u'simple_3.concrete', filenames[2])

    def test_zip_file(self):
        reader = CommunicationReader("tests/testdata/simple.zip")
        [comms, filenames] = zip(*[(c, f) for (c, f) in reader])
        self.assertEqual(u'one', comms[0].id)
        self.assertEqual(u'two', comms[1].id)
        self.assertEqual(u'three', comms[2].id)
        self.assertEqual(u'simple_1.concrete', filenames[0])
        self.assertEqual(u'simple_2.concrete', filenames[1])
        self.assertEqual(u'simple_3.concrete', filenames[2])


class TestCommunicationWriter(unittest.TestCase):
    def test_single_file(self):
        comm = read_communication_from_file("tests/testdata/simple_1.concrete")
        writer = CommunicationWriter()
        (file_handle, filename) = tempfile.mkstemp()
        writer.open(filename)
        writer.write(comm)
        writer.close()

        os.remove(filename)


class TestCommunicationWriterTGZ(unittest.TestCase):
    def test_single_file(self):
        comm = read_communication_from_file("tests/testdata/simple_1.concrete")
        writer = CommunicationWriterTGZ()
        (file_handle, filename) = tempfile.mkstemp()
        writer.open(filename)
        writer.write(comm, "simple_1.concrete")
        writer.close()

        self.assertTrue(tarfile.is_tarfile(filename))
        os.remove(filename)


if __name__ == '__main__':
    unittest.main(buffer=True)



#!/usr/bin/env python

"""Print human-readable information about a Communication to stdout

concrete_inspect.py is a command-line script for printing out information
about a Concrete Communication.
"""

import argparse
import codecs
import sys

import concrete.inspect
import concrete.util


def main():
    # Make stdout output UTF-8, preventing "'ascii' codec can't encode" errors
    sys.stdout = codecs.getwriter('utf8')(sys.stdout)

    parser = argparse.ArgumentParser(description="Print information about a Concrete Communication to stdout")
    parser.add_argument("--char-offsets", help="Print token text extracted from character offsets "
                        "(not the text stored in the tokenization) in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--dependency", help="Print HEAD tags for first dependency parse in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--entities", help="Print info about all Entities and their EntityMentions",
                        action="store_true")
    parser.add_argument("--lemmas", help="Print first set of lemma token tags in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--metadata", help="Print metadata for tools used to annotate Communication",
                        action="store_true")
    parser.add_argument("--mentions", help="Print whitespace-separated tokens, with entity mentions wrapped "
                        "using <ENTITY ID=x> tags, where 'x' is the (zero-indexed) entity number",
                        action="store_true")
    parser.add_argument("--ner", help="Print first set of Named Entity Recognition token tags in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--pos", help="Print first set of Part-Of-Speech token tags in 'ConLL-style' format",
                        action="store_true")
    parser.add_argument("--situation-mentions", help="Print info about all SituationMentions",
                        action="store_true")
    parser.add_argument("--situations", help="Print info about all Situations and their SituationMentions",
                        action="store_true")
    parser.add_argument("--text", help="Print .text field",
                        action="store_true")
    parser.add_argument("--tokens", help="Print whitespace-seperated tokens for *all* Tokenizations in a "
                        "Communication.  Each sentence tokenization is printed on a separate line, and "
                        "empty lines indicate a section break",
                        action="store_true")
    parser.add_argument("--treebank", help="Print Penn-Treebank style parse trees for *all* Constituent "
                        "Parses in the Communication",
                        action="store_true")
    parser.add_argument("communication_file")
    args = parser.parse_args()

    comm = concrete.util.read_communication_from_file(args.communication_file)

    if args.char_offsets or args.dependency or args.lemmas or args.ner or args.pos:
        concrete.inspect.print_conll_style_tags_for_communication(
            comm, char_offsets=args.char_offsets, dependency=args.dependency, lemmas=args.lemmas, ner=args.ner, pos=args.pos)
    elif args.entities:
        concrete.inspect.print_entities(comm)
    elif args.mentions:
        concrete.inspect.print_tokens_with_entityMentions(comm)
    elif args.metadata:
        concrete.inspect.print_metadata(comm)
    elif args.situation_mentions:
        concrete.inspect.print_situation_mentions(comm)
    elif args.situations:
        concrete.inspect.print_situations(comm)
    elif args.text:
        concrete.inspect.print_text_for_communication(comm)
    elif args.tokens:
        concrete.inspect.print_tokens_for_communication(comm)
    elif args.treebank:
        concrete.inspect.print_penn_treebank_for_communication(comm)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class EmailAddress(object):
  """
  An email address, optionally accompanied by a display_name. These
  values are typically extracted from strings such as:
  <tt> "John Smith" &lt;john\@xyz.com&gt; </tt>.

  \see RFC2822 <http://tools.ietf.org/html/rfc2822>

  Attributes:
   - address
   - displayName
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'address', None, None, ), # 1
    (2, TType.STRING, 'displayName', None, None, ), # 2
  )

  def __init__(self, address=None, displayName=None,):
    self.address = address
    self.displayName = displayName

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.address = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.displayName = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('EmailAddress')
    if self.address is not None:
      oprot.writeFieldBegin('address', TType.STRING, 1)
      oprot.writeString(self.address.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.displayName is not None:
      oprot.writeFieldBegin('displayName', TType.STRING, 2)
      oprot.writeString(self.displayName.encode('utf-8'))
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.address)
    value = (value * 31) ^ hash(self.displayName)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class EmailCommunicationInfo(object):
  """
  Extra information about an email communication instance.

  Attributes:
   - messageId
   - contentType
   - userAgent
   - inReplyToList
   - referenceList
   - senderAddress
   - returnPathAddress
   - toAddressList
   - ccAddressList
   - bccAddressList
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'messageId', None, None, ), # 1
    (2, TType.STRING, 'contentType', None, None, ), # 2
    (3, TType.STRING, 'userAgent', None, None, ), # 3
    (4, TType.LIST, 'inReplyToList', (TType.STRING,None), None, ), # 4
    (5, TType.LIST, 'referenceList', (TType.STRING,None), None, ), # 5
    (6, TType.STRUCT, 'senderAddress', (EmailAddress, EmailAddress.thrift_spec), None, ), # 6
    (7, TType.STRUCT, 'returnPathAddress', (EmailAddress, EmailAddress.thrift_spec), None, ), # 7
    (8, TType.LIST, 'toAddressList', (TType.STRUCT,(EmailAddress, EmailAddress.thrift_spec)), None, ), # 8
    (9, TType.LIST, 'ccAddressList', (TType.STRUCT,(EmailAddress, EmailAddress.thrift_spec)), None, ), # 9
    (10, TType.LIST, 'bccAddressList', (TType.STRUCT,(EmailAddress, EmailAddress.thrift_spec)), None, ), # 10
  )

  def __init__(self, messageId=None, contentType=None, userAgent=None, inReplyToList=None, referenceList=None, senderAddress=None, returnPathAddress=None, toAddressList=None, ccAddressList=None, bccAddressList=None,):
    self.messageId = messageId
    self.contentType = contentType
    self.userAgent = userAgent
    self.inReplyToList = inReplyToList
    self.referenceList = referenceList
    self.senderAddress = senderAddress
    self.returnPathAddress = returnPathAddress
    self.toAddressList = toAddressList
    self.ccAddressList = ccAddressList
    self.bccAddressList = bccAddressList

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.messageId = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.contentType = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.userAgent = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.LIST:
          self.inReplyToList = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = iprot.readString().decode('utf-8')
            self.inReplyToList.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.referenceList = []
          (_etype9, _size6) = iprot.readListBegin()
          for _i10 in xrange(_size6):
            _elem11 = iprot.readString().decode('utf-8')
            self.referenceList.append(_elem11)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRUCT:
          self.senderAddress = EmailAddress()
          self.senderAddress.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.STRUCT:
          self.returnPathAddress = EmailAddress()
          self.returnPathAddress.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.LIST:
          self.toAddressList = []
          (_etype15, _size12) = iprot.readListBegin()
          for _i16 in xrange(_size12):
            _elem17 = EmailAddress()
            _elem17.read(iprot)
            self.toAddressList.append(_elem17)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.LIST:
          self.ccAddressList = []
          (_etype21, _size18) = iprot.readListBegin()
          for _i22 in xrange(_size18):
            _elem23 = EmailAddress()
            _elem23.read(iprot)
            self.ccAddressList.append(_elem23)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 10:
        if ftype == TType.LIST:
          self.bccAddressList = []
          (_etype27, _size24) = iprot.readListBegin()
          for _i28 in xrange(_size24):
            _elem29 = EmailAddress()
            _elem29.read(iprot)
            self.bccAddressList.append(_elem29)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('EmailCommunicationInfo')
    if self.messageId is not None:
      oprot.writeFieldBegin('messageId', TType.STRING, 1)
      oprot.writeString(self.messageId.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.contentType is not None:
      oprot.writeFieldBegin('contentType', TType.STRING, 2)
      oprot.writeString(self.contentType.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.userAgent is not None:
      oprot.writeFieldBegin('userAgent', TType.STRING, 3)
      oprot.writeString(self.userAgent.encode('utf-8'))
      oprot.writeFieldEnd()
    if self.inReplyToList is not None:
      oprot.writeFieldBegin('inReplyToList', TType.LIST, 4)
      oprot.writeListBegin(TType.STRING, len(self.inReplyToList))
      for iter30 in self.inReplyToList:
        oprot.writeString(iter30.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.referenceList is not None:
      oprot.writeFieldBegin('referenceList', TType.LIST, 5)
      oprot.writeListBegin(TType.STRING, len(self.referenceList))
      for iter31 in self.referenceList:
        oprot.writeString(iter31.encode('utf-8'))
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.senderAddress is not None:
      oprot.writeFieldBegin('senderAddress', TType.STRUCT, 6)
      self.senderAddress.write(oprot)
      oprot.writeFieldEnd()
    if self.returnPathAddress is not None:
      oprot.writeFieldBegin('returnPathAddress', TType.STRUCT, 7)
      self.returnPathAddress.write(oprot)
      oprot.writeFieldEnd()
    if self.toAddressList is not None:
      oprot.writeFieldBegin('toAddressList', TType.LIST, 8)
      oprot.writeListBegin(TType.STRUCT, len(self.toAddressList))
      for iter32 in self.toAddressList:
        iter32.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.ccAddressList is not None:
      oprot.writeFieldBegin('ccAddressList', TType.LIST, 9)
      oprot.writeListBegin(TType.STRUCT, len(self.ccAddressList))
      for iter33 in self.ccAddressList:
        iter33.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.bccAddressList is not None:
      oprot.writeFieldBegin('bccAddressList', TType.LIST, 10)
      oprot.writeListBegin(TType.STRUCT, len(self.bccAddressList))
      for iter34 in self.bccAddressList:
        iter34.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.messageId)
    value = (value * 31) ^ hash(self.contentType)
    value = (value * 31) ^ hash(self.userAgent)
    value = (value * 31) ^ hash(self.inReplyToList)
    value = (value * 31) ^ hash(self.referenceList)
    value = (value * 31) ^ hash(self.senderAddress)
    value = (value * 31) ^ hash(self.returnPathAddress)
    value = (value * 31) ^ hash(self.toAddressList)
    value = (value * 31) ^ hash(self.ccAddressList)
    value = (value * 31) ^ hash(self.bccAddressList)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



#!/usr/bin/env python

"""Test conversion of Concrete objects to strings
"""

import time
import unittest

from concrete import AnnotationMetadata, Tokenization
from concrete.util import generate_UUID
from test_helper import read_test_comm


class TestRepr(unittest.TestCase):
    def test_repr_on_comm(self):
        """Verify that Communications can be converted to strings.

        Checks for the issue addressed in this commit:

          commit 0ee3317454543b63dc7a273d92e5720bb9210b03
          Author: Craig Harman <craig@craigharman.net>
          Date:   Tue Dec 16 13:08:44 2014 -0500

            Fixed infinite recursion bug in Tokenization.__repr__()

            The addition of an in-memory "backpointer" from a Tokenization to the
            Tokenization's enclosing Sentence inadvertently broke the
            (Thrift auto-generated) Tokenization.__repr__() function.  Modified the
            function to ignore the backpointer when generating the string
            representation for a Tokenization.
        """
        comm = read_test_comm()
        s = comm.__repr__()

    def test_repr_on_tokenization(self):
        """
        """
        tokenization = Tokenization(
            metadata=AnnotationMetadata(tool="test", timestamp=int(time.time())),
            uuid=generate_UUID())
        s = tokenization.__repr__()


if __name__ == '__main__':
    unittest.main(buffer=True)



#!/usr/bin/env python

"""
Command line script to (partially) validate a Concrete Communication

This script is a thin wrapper around the functionality in the
concrete.validate library.
"""

import argparse
import logging

from concrete.validate import validate_communication_file


def main():
    parser = argparse.ArgumentParser(description="Validate a Concrete Communication file")
    parser.add_argument('communication_file')
    parser.add_argument('-l', '--loglevel', choices=['debug', 'info', 'warning', 'error'],
                        help="Log level threshold")
    args = parser.parse_args()

    # Set logging level
    if args.loglevel:
        lowercase_loglevel = args.loglevel.lower()
        if lowercase_loglevel == 'debug':
            loglevel = logging.DEBUG
        elif lowercase_loglevel == 'info':
            loglevel = logging.INFO
        elif lowercase_loglevel == 'warning':
            loglevel = logging.WARNING
        elif lowercase_loglevel == 'error':
            loglevel = logging.ERROR
        else:
            loglevel=logging.DEBUG
    else:
        loglevel=logging.DEBUG
    logging.basicConfig(format='%(levelname)7s:  %(message)s', level=loglevel)

    validate_communication_file(args.communication_file)


if __name__ == "__main__":
    main()



# This file is NOT auto-generated

"""
Python modules and scripts for working with
[Concrete](https://github.com/hltcoe/concrete), an HLT data
specification defined using [Thrift](http://thrift.apache.org).
"""


from audio.ttypes import *
from clustering.ttypes import *
from communication.ttypes import *
from communication_service.ttypes import *
from discourse.ttypes import *
from email.ttypes import *
from entities.ttypes import *
from language.ttypes import *
from linking.ttypes import *
from metadata.ttypes import *
from nitf.ttypes import *
from situations.ttypes import *
from spans.ttypes import *
from structure.ttypes import *
from twitter.ttypes import *
from uuid.ttypes import *

from version import __version__



#!/usr/bin/env python
"""
Compare two Concrete files by converting to JSON then running the Git diff command
"""

import argparse
import codecs
import os
import os.path
import subprocess
import tempfile

from concrete.util import communication_file_to_json, read_communication_from_file


def main():
    parser = argparse.ArgumentParser(description="Compare JSON representation of two concrete files")
    parser.add_argument('file_one')
    parser.add_argument('file_two')
    args = parser.parse_args()

    tmp_path = tempfile.mkdtemp()

    json_one_filename = os.path.join(tmp_path, os.path.basename(args.file_one))
    json_two_filename = os.path.join(tmp_path, os.path.basename(args.file_two))

    # Prevent filename collision
    if json_one_filename == json_two_filename:
        json_two_filename += ".1"

    json_comm_one = communication_file_to_json(args.file_one)
    json_comm_two = communication_file_to_json(args.file_two)
    
    codecs.open(json_one_filename, "w", encoding="utf-8").write(json_comm_one)
    codecs.open(json_two_filename, "w", encoding="utf-8").write(json_comm_two)

    diff_command = "git diff --no-index %s %s" % (json_one_filename, json_two_filename)
    diff_output = subprocess.Popen(diff_command, shell=True, stdout=subprocess.PIPE).stdout.read()

    # Clean up temporary files
    os.remove(json_one_filename)
    os.remove(json_two_filename)
    os.rmdir(tmp_path)

    print diff_output


if __name__ == "__main__":
    main()



"""Library to validate a Concrete Communication

Current validation checks:

  - for each constituent parse, do any of the constituent ID's for
    that parse repeat?
  - is each dependency and constituent parse a fully connected graph?
  - is each constituent parse "tree" really a tree?
  - for each dependency parse, are there any nodes with a null
    governer node whose edgeType is not root?
  - for each entityMention, does entityMention.tokens.tokenizationId
    point to a valid tokenization uuid?
  - for each entity, do all the entity's entityMentionId's point to a
    valid entityMention uuid?
  - for each TokenRefSequence, are the token indices valid indices
    into the corresponding tokenization?
  - for each MentionArgument, there are fields that can point to an
    EntityMention, SituationMention or TokenRefSequence, but the
    MentionArgument should point to exactly one of these objects
"""

import logging

import networkx as nx
from thrift import TSerialization
from thrift.protocol import TProtocol
from thrift.Thrift import TType

from concrete import Communication
from concrete.util import read_communication_from_file



def validate_communication_file(communication_filename):
    logging.info(_ilm(0, "Opening Concrete Communication with filename '%s'" % communication_filename))
    comm = read_communication_from_file(communication_filename)
    validate_communication(comm)


def validate_communication(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - `True` if Communication is valid, `False` otherwise
    """
    valid = True

    logging.info(_ilm(0, "Validating Communication with ID '%s'" % comm.id))

    valid &= validate_thrift_object_required_fields_recursively(comm)

    if comm.sectionList:
        for section in comm.sectionList:
            valid &= validate_token_offsets_for_section(section)
            if section.sentenceList:
                logging.debug(_ilm(4, "section '%s' has %d sentences" %
                                  (section.uuid, len(section.sentenceList))))
                for sentence in section.sentenceList:
                    valid &= validate_token_offsets_for_sentence(sentence)
                    if sentence.tokenization:
                        valid &= validate_constituency_parses(comm, sentence.tokenization)
                        valid &= validate_dependency_parses(sentence.tokenization)
                        valid &= validate_token_taggings(sentence.tokenization)

    valid &= validate_entity_mention_ids(comm)
    valid &= validate_entity_mention_tokenization_ids(comm)
    valid &= validate_entity_mention_token_ref_sequences(comm)
    valid &= validate_situations(comm)
    valid &= validate_situation_mentions(comm)

    if not valid:
        logging.error(_ilm(0, "The Communication with ID '%s' IS NOT valid" % comm.id))
    else:
        logging.info(_ilm(0, "The Communication with ID '%s' is valid" % comm.id))

    return valid


def _get_entity_uuidString_set(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - set of strings: uuidStrings for all Entities in the Communication
    """
    entity_uuidString_set = set()
    if comm.entitySetList:
        for entitySet in comm.entitySetList:
            if entitySet.entityList:
                for entity in entitySet.entityList:
                    entity_uuidString_set.add(entity.uuid.uuidString)
    return entity_uuidString_set


def _get_entity_mention_uuidString_set(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - set of strings: uuidStrings for all EntityMentions in the Communication
    """
    entity_mention_uuidString_set = set()
    if comm.entityMentionSetList:
        for entityMentionSet in comm.entityMentionSetList:
            if entityMentionSet.mentionList:
                for entityMention in entityMentionSet.mentionList:
                    entity_mention_uuidString_set.add(entityMention.uuid.uuidString)
    return entity_mention_uuidString_set


def _get_sentence_for_tokenization_uuidString_dict(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - dictionary mapping of Tokenization uuidStrings to Sentences
    """
    if not hasattr(comm, 'sentence_for_tokenization_uuidString_dict'):
        comm.sentence_for_tokenization_uuidString_dict = {}
        if comm.sectionList:
            for section in comm.sectionList:
                if section.sentenceList:
                    for sentence in section.sentenceList:
                        if sentence.tokenization:
                            comm.sentence_for_tokenization_uuidString_dict[sentence.tokenization.uuid.uuidString] = sentence
    return comm.sentence_for_tokenization_uuidString_dict


def _get_situation_uuidString_set(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - set of strings: uuidStrings for all Situations in the Communication
    """
    situation_uuidString_set = set()
    if comm.situationSetList:
        for situationSet in comm.situationSetList:
            if situationSet.situationList:
                for situation in situationSet.situationList:
                    situation_uuidString_set.add(situation.uuid.uuidString)
    return situation_uuidString_set


def _get_situation_mention_uuidString_set(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - set of strings: uuidStrings for all SituationMentions in the Communication
    """
    situation_mention_uuidString_set = set()
    if comm.situationMentionSetList:
        for situationMentionSet in comm.situationMentionSetList:
            if situationMentionSet.mentionList:
                for situationMention in situationMentionSet.mentionList:
                    situation_mention_uuidString_set.add(situationMention.uuid.uuidString)
    return situation_mention_uuidString_set


def _get_tokenization_uuidString_dict(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - dictionary mapping uuidStrings to Tokenizations
    """
    if not hasattr(comm, '_tokenization_uuidString_dict'):
        comm._tokenization_uuidString_dict = {}
        if comm.sectionList:
            for section in comm.sectionList:
                if section.sentenceList:
                    for sentence in section.sentenceList:
                        if sentence.tokenization:
                            comm._tokenization_uuidString_dict[sentence.tokenization.uuid.uuidString] = sentence.tokenization
    return comm._tokenization_uuidString_dict


def _get_tokenization_uuidString_set(comm):
    """
    Args:

    - `comm` (`Communication`)

    Returns:

    - set of strings: uuidStrings for all Tokenizations in the Communication
    """
    tokenization_uuidString_set = set()
    if comm.sectionList:
        for section in comm.sectionList:
            if section.sentenceList:
                for sentence in section.sentenceList:
                    if sentence.tokenization:
                        tokenization_uuidString_set.add(sentence.tokenization.uuid.uuidString)
    return tokenization_uuidString_set


def _ilm(indent_level, log_message):
    """
    ilm = Indented Log Message

    Prepend spaces to a log message so that log messages can be
    printed in a hierarchical list

    Args:

    - `log_message` (string): Log message to be indented
    - `indent_level` (int): Indentation level

    Returns:

    - string: Indented log message
    """
    return "  " * indent_level + log_message


def validate_constituency_parses(comm, tokenization):
    """
    Args:

    - `comm` (`Communication`)
    - `tokenization` (`Tokenization`)

    Returns:

    - `True` if tokenization's constituency parse is valid, `False` otherwise
    """
    valid = True

    if tokenization.parseList:
        for parse in tokenization.parseList:
            total_constituents = len(parse.constituentList)
            logging.debug(_ilm(6, "tokenization '%s' has %d constituents" % (tokenization.uuid, total_constituents)))

            constituent_id_set = set()
            constituent_parse_tree = nx.DiGraph()

            for constituent in parse.constituentList:
                # Add nodes to parse tree
                constituent_parse_tree.add_node(constituent.id)

                if constituent.id not in constituent_id_set:
                    constituent_id_set.add(constituent.id)
                else:
                    valid = False
                    logging.error(_ilm(7, "constituent ID %d has already been used in this sentence's tokenization" % constituent.id))

            # Add edges to constituent parse tree
            for constituent in parse.constituentList:
                if constituent.childList:
                    for child_id in constituent.childList:
                        constituent_parse_tree.add_edge(constituent.id, child_id)

            # Check if constituent parse "tree" is a fully connected graph
            undirected_graph = constituent_parse_tree.to_undirected()
            if not nx.is_connected(undirected_graph):
                valid = False
                logging.error(_ilm(6, "The constituent parse \"tree\" is not a fully connected graph - the graph has %d components" %
                    nx.number_connected_components(undirected_graph)))

            # Check if constituent parse "tree" is actually a tree
            if nx.number_of_nodes(constituent_parse_tree) != nx.number_of_edges(constituent_parse_tree) + 1:
                valid = False
                logging.error(_ilm(6, "The constituent parse \"tree\" is not a tree.  |V| != |E|+1  (|V|=%d, |E|=%d)" %
                    (nx.number_of_nodes(constituent_parse_tree), nx.number_of_edges(constituent_parse_tree))))

    return valid


def validate_dependency_parses(tokenization):
    """
    Args:

    - `tokenization` (`Tokenization`)

    Returns:

    -  `True` if all of a tokenization's dependency parses are valid, `False` otherwise
    """
    valid = True

    if tokenization.dependencyParseList:
        total_tokens = len(tokenization.tokenList.tokenList)
        for dependencyParse in tokenization.dependencyParseList:
            dependency_parse_tree = nx.DiGraph()

            # Add nodes to dependency parse tree
            for dependency in dependencyParse.dependencyList:
                if dependency.gov is None and dependency.edgeType.lower() != "root":
                    valid = False
                    logging.error(_ilm(7, "Found a null dependency parse node with governer whose edgeType is '%s' instead of 'root'" %
                                          dependency.edgeType))
                if dependency.gov is not None:
                    if dependency.gov < -1 or dependency.gov > total_tokens:
                        valid = False
                        logging.error(_ilm(7, "Found a null dependency parse node with invalid governer of '%d'" %
                                          dependency.gov))
                    dependency_parse_tree.add_node(dependency.gov)
                dependency_parse_tree.add_node(dependency.dep)

            # Add edges to dependency parse tree
            for dependency in dependencyParse.dependencyList:
                if dependency.gov is not None:
                    dependency_parse_tree.add_edge(dependency.gov, dependency.dep)

            # Check if dependency parse "tree" is a fully connected graph
            undirected_graph = dependency_parse_tree.to_undirected()
            try:
                if not nx.is_connected(undirected_graph):
                    valid = False
                    logging.error(_ilm(7, "The dependency parse graph created by '%s' is not a fully connected graph - the graph has %d components" %
                                       (dependencyParse.metadata.tool, nx.number_connected_components(undirected_graph))))
            except nx.exception.NetworkXPointlessConcept:
                logging.warning(_ilm(7, "The dependency parse graph created by '%s' does not have any nodes" %
                                     dependencyParse.metadata.tool))
    return valid


def validate_entity_mention_ids(comm):
    valid = True
    entity_mention_uuidString_set = _get_entity_mention_uuidString_set(comm)

    if comm.entitySetList:
        for entitySet in comm.entitySetList:
            if entitySet.entityList:
                for entity in entitySet.entityList:
                    for entityMentionId in entity.mentionIdList:
                        if entityMentionId.uuidString not in entity_mention_uuidString_set:
                            valid = False
                            logging.error(_ilm(2, "Entity '%s' has an invalid entityMentionId (%s)" % (entity.uuid, entityMentionId)))
    return valid


def validate_entity_mention_tokenization_ids(comm):
    valid = True
    tokenization_uuidString_set = _get_tokenization_uuidString_set(comm)

    if comm.entityMentionSetList:
        for entityMentionSet in comm.entityMentionSetList:
            if entityMentionSet.mentionList:
                for entityMention in entityMentionSet.mentionList:
                    if entityMention.tokens.tokenizationId.uuidString not in tokenization_uuidString_set:
                        valid = False
                        logging.error(_ilm(2, "Mention '%s' has an invalid tokenizationId (%s)" % (entityMention.uuid, entityMention.tokens.tokenizationId)))
    return valid


def validate_entity_mention_token_ref_sequences(comm):
    valid = True
    if comm.entityMentionSetList:
        for entityMentionSet in comm.entityMentionSetList:
            if entityMentionSet.mentionList:
                for entityMention in entityMentionSet.mentionList:
                    valid &= validate_token_ref_sequence(comm, entityMention.tokens)
    return valid


def validate_situation_mentions(comm):
    valid = True
    entity_mention_uuidString_set = _get_entity_mention_uuidString_set(comm)
    situation_mention_uuidString_set = _get_situation_mention_uuidString_set(comm)

    if comm.situationMentionSetList:
        for situationMentionSet in comm.situationMentionSetList:
            if situationMentionSet.mentionList:
                for situationMention in situationMentionSet.mentionList:
                    if situationMention.tokens:
                        valid &= validate_token_ref_sequence(comm, situationMention.tokens)
                    for mention_index, mentionArgument in enumerate(situationMention.argumentList):
                        if mentionArgument.entityMentionId and \
                           mentionArgument.entityMentionId.uuidString not in entity_mention_uuidString_set:
                            valid = False
                            logging.error(_ilm(2, "MentionArgument for SituationMention '%s' has an invalid entityMentionId (%s). Tool='%s'" %
                                              (situationMention.uuid.uuidString, mentionArgument.entityMentionId, situationMentionSet.metadata.tool)))
                        if mentionArgument.situationMentionId and \
                           mentionArgument.situationMentionId.uuidString not in situation_mention_uuidString_set:
                            valid = False
                            logging.error(_ilm(2, "MentionArgument for SituationMention '%s' has an invalid situationMentionId (%s). Tool='%s'" %
                                              (situationMention.uuid, mentionArgument.situationMentionId, situationMentionSet.metadata.tool)))
                        total_args = bool(mentionArgument.tokens) + bool(mentionArgument.entityMentionId) + bool(mentionArgument.situationMentionId)
                        if total_args != 1:
                            valid = False
                            logging.error(_ilm(2, "MentionArgument #%d for SituationMention '%s' should have exactly one EntityMention|SituationMention|TokenRefSequence, but found %d" %
                                              (mention_index, situationMention.uuid.uuidString, total_args)))
    return valid


def validate_situations(comm):
    valid = True

    entity_uuidString_set = _get_entity_uuidString_set(comm)
    situation_mention_uuidString_set = _get_situation_mention_uuidString_set(comm)
    situation_uuidString_set = _get_situation_uuidString_set(comm)

    if comm.situationSetList:
        for situationSet in comm.situationSetList:
            if situationSet.situationList:
                for situation in situationSet.situationList:
                    if situation.argumentList:
                        for argument in situation.argumentList:
                            if argument.situationId and \
                               argument.situationId.uuidString not in situation_uuidString_set:
                                valid = False
                                logging.error(_ilm(2, "Argument for Situation '%s' has an invalid situationId (%s). Tool='%s'" %
                                                  (situation.uuid, argument.situationId, situationSet.metadata.tool)))
                            if argument.entityId and \
                               argument.entityId.uuidString not in entity_uuidString_set:
                                valid = False
                                logging.error(_ilm(2, "Argument for Situation '%s' has an invalid entityId (%s). Tool='%s'" %
                                                  (situation.uuid, argument.entityId, situationSet.metadata.tool)))
                    if situation.justificationList:
                        for justification in situation.justificationList:
                            if justification.mentionId.uuidString not in situation_mention_uuidString_set:
                                valid = False
                                logging.error(_ilm(2, "Justification for Situation '%s' has an invalid [situation] mentionId (%s). Tool='%s'" %
                                                  (situation.uuid, justification.mentionId, situationSet.metadata.tool)))
                            if justification.tokenRefSeqList:
                                for tokenRefSeq in tokenRefSeqList:
                                    valid &= validate_token_ref_sequence(comm, tokenRefSeq)
                    if situation.mentionIdList:
                        for mentionId in situation.mentionIdList:
                            if mentionId.uuidString not in situation_mention_uuidString_set:
                                valid = False
                                logging.error(_ilm(2, "Situation '%s' has an invalid [situation] mentionId (%s). Tool='%s'" %
                                                  (situation.uuid, mentionId, situationSet.metadata.tool)))
    return valid


def validate_token_offsets_for_section(section):
    """
    Test if the TextSpan boundaries for all sentences in a section fall
    within the boundaries of the section's TextSpan
    """
    valid = True

    if section.textSpan == None:
        return valid

    if section.textSpan.start > section.textSpan.ending:
        valid = False
        logging.error(_ilm(2, "Section '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                          (section.uuid, section.textSpan.start, section.textSpan.ending)))

    if section.sentenceList:
        for sentence in section.sentenceList:
            if sentence.textSpan == None:
                continue
            if sentence.textSpan.start > sentence.textSpan.ending:
                valid = False
                logging.error(_ilm(2, "Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                                  (sentence.uuid, sentence.textSpan.start, sentence.textSpan.ending)))
            elif (sentence.textSpan.start < section.textSpan.start) or \
                 (sentence.textSpan.start > section.textSpan.ending) or \
                 (sentence.textSpan.ending < section.textSpan.start) or \
                 (sentence.textSpan.ending > section.textSpan.ending):
                valid = False
                logging.error(_ilm(2, "Sentence '%s' in Section '%s' has a TextSpan [%d, %d] that does not fit within the Section TextSpan [%d, %d]" %
                                  (sentence.uuid, section.uuid, sentence.textSpan.start, sentence.textSpan.ending, section.textSpan.start, section.textSpan.ending)))

    return valid


def validate_token_offsets_for_sentence(sentence):
    """
    Test if the TextSpan boundaries for all tokens in a sentence fall
    within the boundaries of the sentence's TextSpan
    """
    valid = True

    if sentence.textSpan == None:
        return valid

    if sentence.textSpan.start > sentence.textSpan.ending:
        valid = False
        logging.error(_ilm(7, "Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                          (sentence.uuid, sentence.textSpan.start, sentence.textSpan.ending)))
    if sentence.tokenization:
        for token in sentence.tokenization.tokenList.tokenList:
            if token.textSpan == None:
                continue
            if token.textSpan.start > token.textSpan.ending:
                valid = False
                logging.error(_ilm(7, "Token in Sentence '%s' has a TextSpan with a start offset (%d) > end offset (%d)" %
                                  (sentence.uuid, token.textSpan.start, token.textSpan.ending)))
            elif (token.textSpan.start < sentence.textSpan.start) or \
                 (token.textSpan.start > sentence.textSpan.ending) or \
                 (token.textSpan.ending < sentence.textSpan.start) or \
                 (token.textSpan.ending > sentence.textSpan.ending):
                valid = False
                logging.error(_ilm(7, "Token in Sentence '%s' has a TextSpan [%d, %d] that does not fit within the Sentence TextSpan [%d, %d]" %
                                  (sentence.uuid, token.textSpan.start, token.textSpan.ending, sentence.textSpan.start, sentence.textSpan.ending)))

    return valid


def validate_token_ref_sequence(comm, token_ref_sequence):
    valid = True

    tokenization_mapping = _get_tokenization_uuidString_dict(comm)
    sentence_for_tokenization_mapping = _get_sentence_for_tokenization_uuidString_dict(comm)

    if token_ref_sequence.tokenizationId.uuidString not in tokenization_mapping:
        valid = False
        logging.error(_ilm(3, "TokenRefSequence has an invalid tokenizationId (%s)" %
                          token_ref_sequence.tokenizationId.uuidString))
    else:
        tokenization = tokenization_mapping[token_ref_sequence.tokenizationId.uuidString]
        for tokenIndex in token_ref_sequence.tokenIndexList:
            try:
                tokenization.tokenList.tokenList[tokenIndex]
            except IndexError:
                valid = False
                logging.error(_ilm(3, "TokenRefSequence '%s' has an invalid tokenIndex (%d)" %
                                  (token_ref_sequence.tokenizationId.uuidString, tokenIndex)))
    if token_ref_sequence.tokenizationId.uuidString in sentence_for_tokenization_mapping:
        sentence = sentence_for_tokenization_mapping[token_ref_sequence.tokenizationId.uuidString]
        if sentence.textSpan and token_ref_sequence.textSpan:
            if (token_ref_sequence.textSpan.start < sentence.textSpan.start) or \
               (token_ref_sequence.textSpan.start > sentence.textSpan.ending) or \
               (token_ref_sequence.textSpan.ending < sentence.textSpan.start) or \
               (token_ref_sequence.textSpan.ending > sentence.textSpan.ending):
                valid = False
                logging.error(_ilm(2, "TokenRefSequence has a TextSpan [%d, %d] that does not fit within the Sentence TextSpan [%d, %d]" %
                                  (token_ref_sequence.textSpan.start, token_ref_sequence.textSpan.ending, sentence.textSpan.start, sentence.textSpan.ending)))
    return valid


def validate_token_taggings(tokenization):
    """
    Test if a Tokenization has any TokenTaggings with invalid token indices
    """
    valid = True
    if tokenization and tokenization.tokenTaggingList:
        total_tokens = len(tokenization.tokenList.tokenList)
        for token_tagging in tokenization.tokenTaggingList:
            for tagged_token in token_tagging.taggedTokenList:
                if tagged_token.tokenIndex >= total_tokens or tagged_token.tokenIndex < 0:
                    valid = False
                    logging.error(_ilm(7, "TokenTagging '%s' has a tokenIndex '%d' that is out of bounds." %
                                      (token_tagging.uuid.uuidString, tagged_token.tokenIndex)))
    return valid


def validate_thrift_object_required_fields(thrift_object, indent_level=0):
    """
    Test if a thrift object has all required fields.

    This function calls the thrift object's `validate()` function. 
    If an exception is raised because of missing required fields, the
    function catches the exception and logs the exception's error
    message using the Python standard library's `logging` module.

    Args:

    - `thrift_object`
    - `indent_level`: Indentation level for logging error message

    Returns:

    - `True` if the Thrift object has all required fields, `False` otherwise
    """
    try:
        thrift_object.validate()
    except TProtocol.TProtocolException as e:
        thrift_object_name = str(thrift_object.__class__).split('.')[-1].split("'")[0]
        if hasattr(thrift_object, 'uuid') and thrift_object.uuid is not None:
            thrift_object_name += " '%s'" % thrift_object.uuid
        # For readability, add quotes around field name, changing:
        #   Required field id is unset!
        # to:
        #   Required field 'id' is unset!
        em = e.message.replace("Required field ", "Required Field '").replace(" is unset", "' is unset")
        logging.error(_ilm(indent_level, "%s: %s" % (thrift_object_name, em)))
        return False
    else:
        return True



########################################################################################################

def validate_thrift_object_required_fields_recursively(msg, valid=True):
    """Deep validation of thrift messages.

    Args:

    -   `msg`: a Thrift message

    -----

    The Python version of Thrift 0.9.1 does not support deep (recursive)
    validation, and none of the Thrift serialization/deserialization
    code calls even the shallow validation functions provided by Thrift.

    This function implements deep validation.  The code is adapted from:

      https://raw.githubusercontent.com/flamholz/py-thrift-validation-example/master/util/validation.py

    See this blog post for more information:

      http://techblog.ridewithvia.com/post/38231652492/recursive-validation-of-python-thrift-structures

    A patch to implement deep validation was submitted to the Thrift
    repository in February of 2013:

      https://issues.apache.org/jira/browse/THRIFT-1732

    but Thrift 0.9.1 - which was released on 2013-08-21 - does not
    include this functionality.
    """
    assert msg is not None
    valid &= validate_thrift_object_required_fields(msg)

    # Introspect the structure specification.
    # For each field, check type and decide whether to recurse.
    spec = msg.thrift_spec
    for spec_tuple in spec:
        if spec_tuple is None:
            continue

        mtype = spec_tuple[1]
        name = spec_tuple[2]
        attr = getattr(msg, name)
        if not _ShouldRecurse(mtype):
            # Some primitive type that we don't validate.
            continue

        # Fetch the item itself.
        if attr is None:
            continue

        # Field is set and it's a message or collection, so we validate.
        if mtype == TType.STRUCT:
            valid &= validate_thrift_object_required_fields_recursively(attr, valid)
        elif mtype in (TType.LIST, TType.SET):
            subtype = spec_tuple[3][0]
            if _ShouldRecurse(subtype):
                for submsg in attr:
                    valid &= validate_thrift_object_required_fields_recursively(submsg, valid)
        elif mtype == TType.MAP:
            subtype = spec_tuple[3]
            key_type = subtype[0]
            val_type = subtype[2]
            for key, val in attr.iteritems():
                if _ShouldRecurse(key_type):
                    valid &= validate_thrift_object_required_fields_recursively(key, valid)
                if _ShouldRecurse(val_type):
                    valid &= validate_thrift_object_required_fields_recursively(val, valid)

    return valid


_RECURSE_ON = frozenset([TType.STRUCT,
                         TType.LIST,
                         TType.MAP,
                         TType.SET])

def _ShouldRecurse(ttype):
    """Returns True if this ttype is one we recurse on for validation."""
    return ttype in _RECURSE_ON



#!/usr/bin/env python

"""
"""

import unittest

from concrete.util.simple_comm import create_simple_comm

class TestSimpleComm(unittest.TestCase):
    def test_generate_uuid(self):
        comm = create_simple_comm('one')
        self.assertEqual('one', comm.id)


if __name__ == '__main__':
    unittest.main(buffer=True)



"""Helper function for generating a Concrete UUID object
"""

# Force 'import uuid' to import the Python standard library module
# named "uuid", and not the "concrete.uuid" module
from __future__ import absolute_import

import uuid as python_uuid

import concrete

def generate_UUID():
    """Helper function for generating a Concrete UUID object
    """
    c_uuid = concrete.UUID()
    c_uuid.uuidString = str(python_uuid.uuid4())
    return c_uuid



#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class Sound(object):
  """
  A sound wave. A separate optional field is defined for each
  suppported format. Typically, a Sound object will only define
  a single field.

  Note: we may want to have separate fields for separate channels
  (left vs right), etc.

  Attributes:
   - wav
   - mp3
   - sph
   - path: An absolute path to a file on disk where the sound file can be
  found. It is assumed that this path will be accessable from any
  machine that the system is run on (i.e., it should be a shared
  disk, or possibly a mirrored directory).
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'wav', None, None, ), # 1
    (2, TType.STRING, 'mp3', None, None, ), # 2
    (3, TType.STRING, 'sph', None, None, ), # 3
    (4, TType.STRING, 'path', None, None, ), # 4
  )

  def __init__(self, wav=None, mp3=None, sph=None, path=None,):
    self.wav = wav
    self.mp3 = mp3
    self.sph = sph
    self.path = path

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.wav = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.mp3 = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.sph = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.path = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Sound')
    if self.wav is not None:
      oprot.writeFieldBegin('wav', TType.STRING, 1)
      oprot.writeString(self.wav)
      oprot.writeFieldEnd()
    if self.mp3 is not None:
      oprot.writeFieldBegin('mp3', TType.STRING, 2)
      oprot.writeString(self.mp3)
      oprot.writeFieldEnd()
    if self.sph is not None:
      oprot.writeFieldBegin('sph', TType.STRING, 3)
      oprot.writeString(self.sph)
      oprot.writeFieldEnd()
    if self.path is not None:
      oprot.writeFieldBegin('path', TType.STRING, 4)
      oprot.writeString(self.path.encode('utf-8'))
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.wav)
    value = (value * 31) ^ hash(self.mp3)
    value = (value * 31) ^ hash(self.sph)
    value = (value * 31) ^ hash(self.path)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



"""Create a simple (valid) Communication suitable for testing purposes
"""

import time

from concrete import (
    AnnotationMetadata,
    Communication,
    Section,
    Sentence,
    TextSpan,
    Token,
    Tokenization,
    TokenizationKind,
    TokenList
)
from concrete.util import generate_UUID


def create_simple_comm(comm_id, sentence_string="Super simple sentence ."):
    """Create a simple (valid) Communication suitable for testing purposes

    The Communication will have a single Section containing a single
    Sentence.

    Args:

    - `comm_id`: A string specifying a Communication ID
    - `sentence_string`: A string to be used for the sentence text.
       The string will be whitespace-tokenized.

    Returns:

    - A Concrete Communication object
    """
    toolname="TEST"
    timestamp = int(time.time())

    comm = Communication(
        id=comm_id,
        metadata=AnnotationMetadata(tool=toolname, timestamp=timestamp),
        type=toolname,
        uuid=generate_UUID())

    tokenization = Tokenization(
        kind=TokenizationKind.TOKEN_LIST,
        metadata=AnnotationMetadata(tool=toolname, timestamp=timestamp),
        tokenList=TokenList(
            tokenList=[]),
        uuid=generate_UUID())
    token_string_list = sentence_string.split()
    for i, token_string in enumerate(token_string_list):
        tokenization.tokenList.tokenList.append(Token(text=token_string, tokenIndex=i))

    sentence = Sentence(
        textSpan=TextSpan(0, len(sentence_string)),
        tokenization=tokenization,
        uuid=generate_UUID())

    section = Section(
        kind="SectionKind",
        sentenceList=[sentence],
        textSpan=TextSpan(0, len(sentence_string)),
        uuid=generate_UUID())

    comm.sectionList=[section]

    return comm



"""Functions used by `concrete_inspect.py` to print data in a Communication.

The function implementations provide useful examples of how to
interact with many different Concrete datastructures.
"""

from collections import defaultdict
from operator import attrgetter


def print_conll_style_tags_for_communication(comm, char_offsets=False, dependency=False, lemmas=False, ner=False, pos=False):
    """Print 'ConLL-style' tags for the tokens in a Communication

    Args:

    - `comm`: A Concrete Communication object
    - `char_offsets`: A boolean flag for printing token text specified by
          a Token's (optional) TextSpan
    - `dependency`: A boolean flag for printing dependency parse HEAD tags
    - `lemmas`: A boolean flag for printing lemma tags
    - `ner`: A boolean flag for printing Named Entity Recognition tags
    - `pos`: A boolean flag for printing Part-of-Speech tags
    """
    header_fields = [u"INDEX", u"TOKEN"]
    if char_offsets:
        header_fields.append(u"CHAR")
    if lemmas:
        header_fields.append(u"LEMMA")
    if pos:
        header_fields.append(u"POS")
    if ner:
        header_fields.append(u"NER")
    if dependency:
        header_fields.append(u"HEAD")
    print u"\t".join(header_fields)
    dashes = ["-"*len(fieldname) for fieldname in header_fields]
    print u"\t".join(dashes)

    for tokenization in get_tokenizations(comm):
        token_tag_lists = []

        if char_offsets:
            token_tag_lists.append(get_char_offset_tags_for_tokenization(comm, tokenization))
        if lemmas:
            token_tag_lists.append(get_lemma_tags_for_tokenization(tokenization))
        if pos:
            token_tag_lists.append(get_pos_tags_for_tokenization(tokenization))
        if ner:
            token_tag_lists.append(get_ner_tags_for_tokenization(tokenization))
        if dependency:
            token_tag_lists.append(get_conll_head_tags_for_tokenization(tokenization))
        print_conll_style_tags_for_tokenization(tokenization, token_tag_lists)
        print


def print_conll_style_tags_for_tokenization(tokenization, token_tag_lists):
    """Print 'ConLL-style' tags for the tokens in a tokenization

    Args:

    - `tokenization`: A Concrete Tokenization object
    - `token_tag_lists`: A list of lists of token tag strings
    """
    if tokenization.tokenList:
        for i, token in enumerate(tokenization.tokenList.tokenList):
            token_tags = [unicode(token_tag_list[i]) for token_tag_list in token_tag_lists]
            fields = [unicode(i+1), token.text]
            fields.extend(token_tags)
            print u"\t".join(fields)


def print_entities(comm):
    """Print information for all Entities and their EntityMentions

    Args:

    - `comm`: A Concrete Communication object
    """
    if comm.entitySetList:
        for entitySet_index, entitySet in enumerate(comm.entitySetList):
            if entitySet.metadata:
                print u"Entity Set %d (%s):" % (entitySet_index, entitySet.metadata.tool)
            else:
                print u"Entity Set %d:" % entitySet_index
            for entity_index, entity in enumerate(entitySet.entityList):
                print u"  Entity %d-%d:" % (entitySet_index, entity_index)
                for entityMention_index, entityMention in enumerate(entity.mentionList):
                    print u"      EntityMention %d-%d-%d:" % (entitySet_index, entity_index, entityMention_index)
                    print u"          tokens:     %s" % u" ".join(get_tokens_for_entityMention(entityMention))
                    if entityMention.text:
                        print u"          text:       %s" % entityMention.text
                    print u"          entityType: %s" % entityMention.entityType
                    print u"          phraseType: %s" % entityMention.phraseType
                print
            print



def print_metadata(comm):
    """Print metadata for tools used to annotate Communication
    """
    def _get_tokenizations(comm):
        tokenizations = []
        if comm.sectionList:
            for section in comm.sectionList:
                if section.sentenceList:
                    for sentence in section.sentenceList:
                        if sentence.tokenization:
                            tokenizations.append(sentence.tokenization)
        return tokenizations

    print u"Communication:  %s\n" % comm.metadata.tool

    dependency_parse_tools = set()
    parse_tools = set()
    tokenization_tools = set()
    token_tagging_tools = set()
    for tokenization in _get_tokenizations(comm):
        tokenization_tools.add(tokenization.metadata.tool)
        if tokenization.tokenTaggingList:
            for tokenTagging in tokenization.tokenTaggingList:
                token_tagging_tools.add(tokenTagging.metadata.tool)
        if tokenization.dependencyParseList:
            for dependencyParse in tokenization.dependencyParseList:
                dependency_parse_tools.add(dependencyParse.metadata.tool)
        if tokenization.parseList:
            for parse in tokenization.parseList:
                parse_tools.add(parse.metadata.tool)

    if tokenization_tools:
        for toolname in sorted(tokenization_tools):
            print u"  Tokenization:  %s" % toolname
        print
    if dependency_parse_tools:
        for toolname in sorted(dependency_parse_tools):
            print u"    Dependency Parse:  %s" % toolname
        print
    if parse_tools:
        for toolname in sorted(parse_tools):
            print u"    Parse:  %s" % toolname
        print
    if token_tagging_tools:
        for toolname in sorted(token_tagging_tools):
            print u"    TokenTagging:  %s" % toolname
        print

    if comm.entityMentionSetList:
        for i, entityMentionSet in enumerate(comm.entityMentionSetList):
            print u"  EntityMentionSet #%d:  %s" % (i, entityMentionSet.metadata.tool)
        print
    if comm.entitySetList:
        for i, entitySet in enumerate(comm.entitySetList):
            print u"  EntitySet #%d:  %s" % (i, entitySet.metadata.tool)
        print
    if comm.situationMentionSetList:
        for i, situationMentionSet in enumerate(comm.situationMentionSetList):
            print u"  SituationMentionSet #%d:  %s" % (i, situationMentionSet.metadata.tool)
        print
    if comm.situationSetList:
        for i, situationSet in enumerate(comm.situationSetList):
            print u"  SituationSet #%d:  %s" % (i, situationSet.metadata.tool)
        print


def print_situation_mentions(comm):
    """Print information for all SituationMentions (some of which may not have Situations)

    Args:

    - `comm`: A Concrete Communication
    """
    if comm.situationMentionSetList:
        for situationMentionSet_index, situationMentionSet in enumerate(comm.situationMentionSetList):
            if situationMentionSet.metadata:
                print u"Situation Set %d (%s):" % (situationMentionSet_index, situationMentionSet.metadata.tool)
            else:
                print u"Situation Set %d:" % situationMentionSet_index
            for situationMention_index, situationMention in enumerate(situationMentionSet.mentionList):
                print u"  SituationMention %d-%d:" % (situationMentionSet_index, situationMention_index)
                _print_situation_mention(situationMention)
                print
            print


def print_situations(comm):
    """Print information for all Situations and their SituationMentions

    Args:

    - `comm`: A Concrete Communication
    """
    if comm.situationSetList:
        for situationSet_index, situationSet in enumerate(comm.situationSetList):
            if situationSet.metadata:
                print u"Situation Set %d (%s):" % (situationSet_index, situationSet.metadata.tool)
            else:
                print u"Situation Set %d:" % situationSet_index
            for situation_index, situation in enumerate(situationSet.situationList):
                print u"  Situation %d-%d:" % (situationSet_index, situation_index)
                _p(6, 18, u"situationType", situation.situationType)
                if situation.mentionList:
                    for situationMention_index, situationMention in enumerate(situation.mentionList):
                        print u" "*6 + u"SituationMention %d-%d-%d:" % (
                            situationSet_index, situation_index, situationMention_index)
                        _print_situation_mention(situationMention)
                print
            print


def _print_situation_mention(situationMention):
    """Helper function for printing info for a SituationMention"""
    if situationMention.text:
        _p(10, 20, u"text", situationMention.text)
    if situationMention.situationType:
        _p(10, 20, u"situationType", situationMention.situationType)
    if situationMention.argumentList:
        for argument_index, mentionArgument in enumerate(situationMention.argumentList):
            print u" "*10 + u"Argument %d:" % argument_index
            if mentionArgument.role:
                _p(14, 16, u"role", mentionArgument.role)
            if mentionArgument.entityMention:
                _p(14, 16, u"entityMention",
                    u" ".join(get_tokens_for_entityMention(mentionArgument.entityMention)))
            # A SituationMention can have an argumentList with a MentionArgument that
            # points to another SituationMention - which could conceivably lead to
            # loops.  We currently don't traverse the list recursively, instead looking
            # at only SituationMentions referenced by top-level SituationMentions
            if mentionArgument.situationMention:
                print u" "*14 + u"situationMention:"
                if situationMention.text:
                    _p(18, 20, u"text", situationMention.text)
                if situationMention.situationType:
                    _p(18, 20, u"situationType", situationMention.situationType)


def _p(indent_level, justified_width, fieldname, content):
    """Text alignment helper function"""
    print u" "*indent_level + (fieldname + u":").ljust(justified_width) + content


def print_text_for_communication(comm):
    print comm.text


def print_tokens_with_entityMentions(comm):
    entityMentions_by_tokenizationId = get_entityMentions_by_tokenizationId(comm)
    entity_number_for_entityMention_uuid = get_entity_number_for_entityMention_uuid(comm)
    tokenizations_by_section = get_tokenizations_grouped_by_section(comm)

    for tokenizations_in_section in tokenizations_by_section:
        for tokenization in tokenizations_in_section:
            if tokenization.tokenList:
                text_tokens = [token.text for token in tokenization.tokenList.tokenList]
                if tokenization.uuid.uuidString in entityMentions_by_tokenizationId:
                    for entityMention in entityMentions_by_tokenizationId[tokenization.uuid.uuidString]:
                        # TODO: Handle non-contiguous tokens in a tokenIndexLists
                        first_token_index = entityMention.tokens.tokenIndexList[0]
                        last_token_index = entityMention.tokens.tokenIndexList[-1]
                        entity_number = entity_number_for_entityMention_uuid[entityMention.uuid.uuidString]
                        text_tokens[first_token_index] = u"<ENTITY ID=%d>%s" % (entity_number, text_tokens[first_token_index])
                        text_tokens[last_token_index] = u"%s</ENTITY>" % text_tokens[last_token_index]
                print u" ".join(text_tokens)
        print


def print_tokens_for_communication(comm):
    """
    """
    tokenizations_by_section = get_tokenizations_grouped_by_section(comm)

    for tokenizations_in_section in tokenizations_by_section:
        for tokenization in tokenizations_in_section:
            if tokenization.tokenList:
                text_tokens = [token.text for token in tokenization.tokenList.tokenList]
                print u" ".join(text_tokens)
        print


def print_penn_treebank_for_communication(comm):
    """Print Penn-Treebank parse trees for all tokenizations

    Args:

    - `comm`: A Concrete Communication object
    """
    tokenizations = get_tokenizations(comm)

    for tokenization in tokenizations:
        if tokenization.parseList:
            for parse in tokenization.parseList:
                print penn_treebank_for_parse(parse) + u"\n\n"


def penn_treebank_for_parse(parse):
    """Get a Penn-Treebank style string for a Concrete Parse object

    Args:

    - `parse`: A Concrete Parse object

    Returns:

    - A string containing a Penn Treebank style parse tree representation
    """
    def _traverse_parse(nodes, node_index, indent=0):
        s = u""
        indent += len(nodes[node_index].tag) + 2
        if nodes[node_index].childList:
            s += u"(%s " % nodes[node_index].tag
            for i, child_node_index in enumerate(nodes[node_index].childList):
                if i > 0:
                    s += u"\n" + u" "*indent
                s += _traverse_parse(nodes, child_node_index, indent)
            s += u")"
        else:
            s += nodes[node_index].tag
        return s

    sorted_nodes = sorted(parse.constituentList, key=attrgetter('id'))
    return _traverse_parse(sorted_nodes, 0)


def get_char_offset_tags_for_tokenization(comm, tokenization):
    # TODOC
    if tokenization.tokenList:
        char_offset_tags = [None]*len(tokenization.tokenList.tokenList)

        if comm.text:
            for i, token in enumerate(tokenization.tokenList.tokenList):
                if token.textSpan:
                    char_offset_tags[i] = comm.text[token.textSpan.start:token.textSpan.ending]
        return char_offset_tags


def get_conll_head_tags_for_tokenization(tokenization, dependency_parse_index=0):
    """Get a list of ConLL 'HEAD tags' for a tokenization

    In the ConLL data format:

        http://ufal.mff.cuni.cz/conll2009-st/task-description.html

    the HEAD for a token is the (1-indexed) index of that token's
    parent token.  The root token of the dependency parse has a HEAD
    index of 0.

    Args:

    - `tokenization`: A Concrete Tokenization object

    Returns:

    - A list of ConLL 'HEAD tag' strings, with one HEAD tag for each
      token in the supplied tokenization.  If a token does not have
      a HEAD tag (e.g. punctuation tokens), the HEAD tag is an empty
      string.

      If the tokenization does not have a Dependency Parse, this
      function returns a list of empty strings for each token in the
      supplied tokenization.
    """
    if tokenization.tokenList:
        # Tokens that are not part of the dependency parse
        # (e.g. punctuation) are represented using an empty string
        head_list = [""]*len(tokenization.tokenList.tokenList)

        if tokenization.dependencyParseList:
            for dependency in tokenization.dependencyParseList[dependency_parse_index].dependencyList:
                if dependency.gov is None:
                    head_list[dependency.dep] = 0
                else:
                    head_list[dependency.dep] = dependency.gov + 1
        return head_list
    else:
        return []


def get_entityMentions_by_tokenizationId(comm):
    """Get entity mentions for a Communication grouped by Tokenization UUID string

    Args:

    - `comm`: A Concrete Communication object

    Returns:

    - A dictionary of lists of EntityMentions, where the dictionary
      keys are Tokenization UUID strings.
    """
    mentions_by_tokenizationId = defaultdict(list)
    if comm.entitySetList:
        for entitySet in comm.entitySetList:
            for entity in entitySet.entityList:
                for entityMention in entity.mentionList:
                    mentions_by_tokenizationId[entityMention.tokens.tokenizationId.uuidString].append(entityMention)
    return mentions_by_tokenizationId


def get_entity_number_for_entityMention_uuid(comm):
    """Create mapping from EntityMention UUID to (zero-indexed) 'Entity Number'

    Args:

    - `comm`: A Concrete Communication object

    Returns:

    - A dictionary where the keys are EntityMention UUID strings,
      and the values are "Entity Numbers", where the first Entity is
      assigned number 0, the second Entity is assigned number 1,
      etc.
    """
    entity_number_for_entityMention_uuid = {}
    entity_number_counter = 0

    if comm.entitySetList:
        for entitySet in comm.entitySetList:
            for entity in entitySet.entityList:
                for entityMention in entity.mentionList:
                    entity_number_for_entityMention_uuid[entityMention.uuid.uuidString] = entity_number_counter
                entity_number_counter += 1
    return entity_number_for_entityMention_uuid


def get_lemma_tags_for_tokenization(tokenization, lemma_tokentagging_index=0):
    """Get lemma tags for a tokenization

    Args:

    - `tokenization`: A Concrete Tokenization object

    Returns:

    - A list of lemma tags for each token in the Tokenization
    """
    if tokenization.tokenList:
        lemma_tags = [""]*len(tokenization.tokenList.tokenList)
        lemma_tokentaggings = get_tokentaggings_of_type(tokenization, u"lemma")
        if lemma_tokentaggings and len(lemma_tokentaggings) > lemma_tokentagging_index:
            tag_for_tokenIndex = {}
            for taggedToken in lemma_tokentaggings[lemma_tokentagging_index].taggedTokenList:
                tag_for_tokenIndex[taggedToken.tokenIndex] = taggedToken.tag
            for i, token in enumerate(tokenization.tokenList.tokenList):
                if i in tag_for_tokenIndex:
                    lemma_tags[i] = tag_for_tokenIndex[i]
        return lemma_tags


def get_ner_tags_for_tokenization(tokenization, ner_tokentagging_index=0):
    """Get Named Entity Recognition tags for a tokenization

    Args:

    - `tokenization`: A Concrete Tokenization object

    Returns:

    - A list of NER tags for each token in the Tokenization
    """
    if tokenization.tokenList:
        ner_tags = [""]*len(tokenization.tokenList.tokenList)
        ner_tokentaggings = get_tokentaggings_of_type(tokenization, u"NER")
        if ner_tokentaggings and len(ner_tokentaggings) > ner_tokentagging_index:
            tag_for_tokenIndex = {}
            for taggedToken in ner_tokentaggings[ner_tokentagging_index].taggedTokenList:
                tag_for_tokenIndex[taggedToken.tokenIndex] = taggedToken.tag
            for i, token in enumerate(tokenization.tokenList.tokenList):
                try:
                    ner_tags[i] = tag_for_tokenIndex[i]
                except IndexError:
                    ner_tags[i] = u""
                if ner_tags[i] == u"NONE":
                    ner_tags[i] = u""
        return ner_tags


def get_pos_tags_for_tokenization(tokenization, pos_tokentagging_index=0):
    """Get Part-of-Speech tags for a tokenization

    Args:

    - tokenization: A Concrete Tokenization object

    Returns:

    - A list of POS tags for each token in the Tokenization
    """
    if tokenization.tokenList:
        pos_tags = [""]*len(tokenization.tokenList.tokenList)
        pos_tokentaggings = get_tokentaggings_of_type(tokenization, u"POS")
        if pos_tokentaggings and len(pos_tokentaggings) > pos_tokentagging_index:
            tag_for_tokenIndex = {}
            for taggedToken in pos_tokentaggings[pos_tokentagging_index].taggedTokenList:
                tag_for_tokenIndex[taggedToken.tokenIndex] = taggedToken.tag
            for i, token in enumerate(tokenization.tokenList.tokenList):
                if i in tag_for_tokenIndex:
                    pos_tags[i] = tag_for_tokenIndex[i]
        return pos_tags


def get_tokenizations(comm):
    """Returns a flat list of all Tokenization objects in a Communication

    Args:

    - `comm`: A Concrete Communication

    Returns:

    - A list of all Tokenization objects within the Communication
    """
    tokenizations = []

    if comm.sectionList:
        for section in comm.sectionList:
            if section.sentenceList:
                for sentence in section.sentenceList:
                    if sentence.tokenization:
                        tokenizations.append(sentence.tokenization)
    return tokenizations


def get_tokenizations_grouped_by_section(comm):
    """Returns a list of lists of Tokenization objects in a Communication

    Args:

    - `comm`: A Concrete Communication

    Returns:

    - Returns a list of lists of Tokenization objects, where the
      Tokenization objects are grouped by Section
    """
    tokenizations_by_section = []

    if comm.sectionList:
        for section in comm.sectionList:
            tokenizations_in_section = []
            if section.sentenceList:
                for sentence in section.sentenceList:
                    if sentence.tokenization:
                        tokenizations_in_section.append(sentence.tokenization)
            tokenizations_by_section.append(tokenizations_in_section)

    return tokenizations_by_section


def get_tokens_for_entityMention(entityMention):
    """Get list of token strings for an EntityMention

    Args:

    - `entityMention`: A Concrete EntityMention argument

    Returns:

    - A list of token strings
    """
    tokens = []
    for tokenIndex in entityMention.tokens.tokenIndexList:
        tokens.append(entityMention.tokens.tokenization.tokenList.tokenList[tokenIndex].text)
    return tokens


def get_tokentaggings_of_type(tokenization, taggingType):
    """Returns a list of TokenTagging objects with the specified taggingType

    Args:

    - `tokenization`: A Concrete Tokenizaiton object
    - `taggingType`: A string value for the specified TokenTagging.taggingType

    Returns:

    - A list of TokenTagging objects
    """
    return [tt for tt in tokenization.tokenTaggingList if tt.taggingType.lower() == taggingType.lower()]



#
# Autogenerated by Thrift Compiler (0.9.1)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None



class UUID(object):
  """
  Attributes:
   - uuidString: A string representation of a UUID, in the format of:
  <pre>
  550e8400-e29b-41d4-a716-446655440000
  </pre>
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'uuidString', None, None, ), # 1
  )

  def __init__(self, uuidString=None,):
    self.uuidString = uuidString

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.uuidString = iprot.readString().decode('utf-8')
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('UUID')
    if self.uuidString is not None:
      oprot.writeFieldBegin('uuidString', TType.STRING, 1)
      oprot.writeString(self.uuidString.encode('utf-8'))
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    if self.uuidString is None:
      raise TProtocol.TProtocolException(message='Required field uuidString is unset!')
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.uuidString)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)
